@@ -1,27 +1,5 @@
 package SQL::Translator::Schema;
 
-# vim: sw=4: ts=4:
-
-# ----------------------------------------------------------------------
-# $Id: Schema.pm,v 1.27 2007-10-24 10:58:35 schiffbruechige Exp $
-# ----------------------------------------------------------------------
-# Copyright (C) 2002-4 SQLFairy Authors
-#
-# This program is free software; you can redistribute it and/or
-# modify it under the terms of the GNU General Public License as
-# published by the Free Software Foundation; version 2.
-#
-# This program is distributed in the hope that it will be useful, but
-# WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-# 02111-1307  USA
-# -------------------------------------------------------------------
-
 =pod
 
 =head1 NAME
@@ -48,39 +26,46 @@ returns the database structure.
 
 =cut
 
-use strict;
+use Moo 1.000003;
 use SQL::Translator::Schema::Constants;
 use SQL::Translator::Schema::Procedure;
 use SQL::Translator::Schema::Table;
 use SQL::Translator::Schema::Trigger;
 use SQL::Translator::Schema::View;
-use SQL::Translator::Schema::Graph;
+use Sub::Quote qw(quote_sub);
+
 use SQL::Translator::Utils 'parse_list_arg';
+use Carp;
 
-use base 'SQL::Translator::Schema::Object';
-use vars qw[ $VERSION $TABLE_ORDER $VIEW_ORDER $TRIGGER_ORDER $PROC_ORDER ];
+extends 'SQL::Translator::Schema::Object';
 
-$VERSION = sprintf "%d.%02d", q$Revision: 1.27 $ =~ /(\d+)\.(\d+)/;
+our $VERSION = '1.59';
 
-__PACKAGE__->_attributes(qw/name database translator/);
 
-# ----------------------------------------------------------------------
+has _order => (is => 'ro', default => quote_sub(q{ +{ map { $_ => 0 } qw/
+    table
+    view
+    trigger
+    proc
+  /} }),
+);
+
+# FIXME - to be removed, together with the SQL::Translator::Schema::Graph* stuff
+# looks like a remnant of the Turnkey project integration back in 2003-4
+# Appears to be quite dead
 sub as_graph {
 
-=pod
+    eval { require Class::MakeMethods }
+      or croak 'You need to install the CPAN dependency Class::MakeMethods to use as_graph()';
 
-=head2 as_graph
-
-Returns the schema as an L<SQL::Translator::Schema::Graph> object.
-
-=cut
+    require  SQL::Translator::Schema::Graph;
 
     my $self = shift;
+
     return SQL::Translator::Schema::Graph->new(
         translator => $self->translator );
 }
 
-# ----------------------------------------------------------------------
 sub as_graph_pm {
 
 =pod
@@ -95,11 +80,11 @@ Returns a Graph::Directed object with the table names for nodes.
 
     my $self = shift;
     my $g    = Graph::Directed->new;
-    
-    for my $table ( $self->get_tables ) { 
+
+    for my $table ( $self->get_tables ) {
         my $tname  = $table->name;
         $g->add_vertex( $tname );
-    
+
         for my $field ( $table->get_fields ) {
             if ( $field->is_foreign_key ) {
                 my $fktable = $field->foreign_key_reference->reference_table;
@@ -112,7 +97,8 @@ Returns a Graph::Directed object with the table names for nodes.
     return $g;
 }
 
-# ----------------------------------------------------------------------
+has _tables => ( is => 'ro', init_arg => undef, default => quote_sub(q{ +{} }) );
+
 sub add_table {
 
 =pod
@@ -121,7 +107,7 @@ sub add_table {
 
 Add a table object.  Returns the new SQL::Translator::Schema::Table object.
 The "name" parameter is required.  If you try to create a table with the
-same name as an existing table, you will get an error and the table will 
+same name as an existing table, you will get an error and the table will
 not be created.
 
   my $t1 = $schema->add_table( name => 'foo' ) or die $schema->error;
@@ -139,28 +125,27 @@ not be created.
         $table->schema($self);
     }
     else {
-        my %args = @_;
+        my %args = ref $_[0] eq 'HASH' ? %{ $_[0] } : @_;
         $args{'schema'} = $self;
         $table = $table_class->new( \%args )
           or return $self->error( $table_class->error );
     }
 
-    $table->order( ++$TABLE_ORDER );
+    $table->order( ++$self->_order->{table} );
 
     # We know we have a name as the Table->new above errors if none given.
     my $table_name = $table->name;
 
-    if ( defined $self->{'tables'}{$table_name} ) {
-        return $self->error(qq[Can't create table: "$table_name" exists]);
+    if ( defined $self->_tables->{$table_name} ) {
+        return $self->error(qq[Can't use table name "$table_name": table exists]);
     }
     else {
-        $self->{'tables'}{$table_name} = $table;
+        $self->_tables->{$table_name} = $table;
     }
 
     return $table;
 }
 
-# ----------------------------------------------------------------------
 sub drop_table {
 
 =pod
@@ -190,22 +175,23 @@ can be set to 1 to also drop all triggers on the table, default is 0.
     my %args    = @_;
     my $cascade = $args{'cascade'};
 
-    if ( !exists $self->{'tables'}{$table_name} ) {
+    if ( !exists $self->_tables->{$table_name} ) {
         return $self->error(qq[Can't drop table: $table_name" doesn't exist]);
     }
 
-    my $table = delete $self->{'tables'}{$table_name};
+    my $table = delete $self->_tables->{$table_name};
 
     if ($cascade) {
 
         # Drop all triggers on this table
         $self->drop_trigger()
-          for ( grep { $_->on_table eq $table_name } @{ $self->{'triggers'} } );
+          for ( grep { $_->on_table eq $table_name } values %{ $self->_triggers } );
     }
     return $table;
 }
 
-# ----------------------------------------------------------------------
+has _procedures => ( is => 'ro', init_arg => undef, default => quote_sub(q{ +{} }) );
+
 sub add_procedure {
 
 =pod
@@ -232,29 +218,28 @@ procedure will not be created.
         $procedure->schema($self);
     }
     else {
-        my %args = @_;
+        my %args = ref $_[0] eq 'HASH' ? %{ $_[0] } : @_;
         $args{'schema'} = $self;
         return $self->error('No procedure name') unless $args{'name'};
         $procedure = $procedure_class->new( \%args )
           or return $self->error( $procedure_class->error );
     }
 
-    $procedure->order( ++$PROC_ORDER );
+    $procedure->order( ++$self->_order->{proc} );
     my $procedure_name = $procedure->name
       or return $self->error('No procedure name');
 
-    if ( defined $self->{'procedures'}{$procedure_name} ) {
+    if ( defined $self->_procedures->{$procedure_name} ) {
         return $self->error(
             qq[Can't create procedure: "$procedure_name" exists] );
     }
     else {
-        $self->{'procedures'}{$procedure_name} = $procedure;
+        $self->_procedures->{$procedure_name} = $procedure;
     }
 
     return $procedure;
 }
 
-# ----------------------------------------------------------------------
 sub drop_procedure {
 
 =pod
@@ -281,17 +266,18 @@ object.
         $proc_name = shift;
     }
 
-    if ( !exists $self->{'procedures'}{$proc_name} ) {
+    if ( !exists $self->_procedures->{$proc_name} ) {
         return $self->error(
             qq[Can't drop procedure: $proc_name" doesn't exist]);
     }
 
-    my $proc = delete $self->{'procedures'}{$proc_name};
+    my $proc = delete $self->_procedures->{$proc_name};
 
     return $proc;
 }
 
-# ----------------------------------------------------------------------
+has _triggers => ( is => 'ro', init_arg => undef, default => quote_sub(q{ +{} }) );
+
 sub add_trigger {
 
 =pod
@@ -300,7 +286,7 @@ sub add_trigger {
 
 Add a trigger object.  Returns the new SQL::Translator::Schema::Trigger object.
 The "name" parameter is required.  If you try to create a trigger with the
-same name as an existing trigger, you will get an error and the trigger will 
+same name as an existing trigger, you will get an error and the trigger will
 not be created.
 
   my $t1 = $schema->add_trigger( name => 'foo' );
@@ -318,27 +304,26 @@ not be created.
         $trigger->schema($self);
     }
     else {
-        my %args = @_;
+        my %args = ref $_[0] eq 'HASH' ? %{ $_[0] } : @_;
         $args{'schema'} = $self;
         return $self->error('No trigger name') unless $args{'name'};
         $trigger = $trigger_class->new( \%args )
           or return $self->error( $trigger_class->error );
     }
 
-    $trigger->order( ++$TRIGGER_ORDER );
+    $trigger->order( ++$self->_order->{trigger} );
 
     my $trigger_name = $trigger->name or return $self->error('No trigger name');
-    if ( defined $self->{'triggers'}{$trigger_name} ) {
+    if ( defined $self->_triggers->{$trigger_name} ) {
         return $self->error(qq[Can't create trigger: "$trigger_name" exists]);
     }
     else {
-        $self->{'triggers'}{$trigger_name} = $trigger;
+        $self->_triggers->{$trigger_name} = $trigger;
     }
 
     return $trigger;
 }
 
-# ----------------------------------------------------------------------
 sub drop_trigger {
 
 =pod
@@ -364,17 +349,18 @@ trigger name or an C<SQL::Translator::Schema::Trigger> object.
         $trigger_name = shift;
     }
 
-    if ( !exists $self->{'triggers'}{$trigger_name} ) {
+    if ( !exists $self->_triggers->{$trigger_name} ) {
         return $self->error(
             qq[Can't drop trigger: $trigger_name" doesn't exist]);
     }
 
-    my $trigger = delete $self->{'triggers'}{$trigger_name};
+    my $trigger = delete $self->_triggers->{$trigger_name};
 
     return $trigger;
 }
 
-# ----------------------------------------------------------------------
+has _views => ( is => 'ro', init_arg => undef, default => quote_sub(q{ +{} }) );
+
 sub add_view {
 
 =pod
@@ -383,7 +369,7 @@ sub add_view {
 
 Add a view object.  Returns the new SQL::Translator::Schema::View object.
 The "name" parameter is required.  If you try to create a view with the
-same name as an existing view, you will get an error and the view will 
+same name as an existing view, you will get an error and the view will
 not be created.
 
   my $v1 = $schema->add_view( name => 'foo' );
@@ -401,26 +387,25 @@ not be created.
         $view->schema($self);
     }
     else {
-        my %args = @_;
+        my %args = ref $_[0] eq 'HASH' ? %{ $_[0] } : @_;
         $args{'schema'} = $self;
         return $self->error('No view name') unless $args{'name'};
         $view = $view_class->new( \%args ) or return $view_class->error;
     }
 
-    $view->order( ++$VIEW_ORDER );
+    $view->order( ++$self->_order->{view} );
     my $view_name = $view->name or return $self->error('No view name');
 
-    if ( defined $self->{'views'}{$view_name} ) {
+    if ( defined $self->_views->{$view_name} ) {
         return $self->error(qq[Can't create view: "$view_name" exists]);
     }
     else {
-        $self->{'views'}{$view_name} = $view;
+        $self->_views->{$view_name} = $view;
     }
 
     return $view;
 }
 
-# ----------------------------------------------------------------------
 sub drop_view {
 
 =pod
@@ -446,20 +431,15 @@ name or an C<SQL::Translator::Schema::View> object.
         $view_name = shift;
     }
 
-    if ( !exists $self->{'views'}{$view_name} ) {
+    if ( !exists $self->_views->{$view_name} ) {
         return $self->error(qq[Can't drop view: $view_name" doesn't exist]);
     }
 
-    my $view = delete $self->{'views'}{$view_name};
+    my $view = delete $self->_views->{$view_name};
 
     return $view;
 }
 
-# ----------------------------------------------------------------------
-sub database {
-
-=pod
-
 =head2 database
 
 Get or set the schema's database.  (optional)
@@ -468,12 +448,8 @@ Get or set the schema's database.  (optional)
 
 =cut
 
-    my $self = shift;
-    $self->{'database'} = shift if @_;
-    return $self->{'database'} || '';
-}
+has database => ( is => 'rw', default => quote_sub(q{ '' }) );
 
-# ----------------------------------------------------------------------
 sub is_valid {
 
 =pod
@@ -497,7 +473,6 @@ Returns true if all the tables and views are valid.
     return 1;
 }
 
-# ----------------------------------------------------------------------
 sub get_procedure {
 
 =pod
@@ -513,11 +488,10 @@ Returns a procedure by the name provided.
     my $self = shift;
     my $procedure_name = shift or return $self->error('No procedure name');
     return $self->error(qq[Table "$procedure_name" does not exist])
-      unless exists $self->{'procedures'}{$procedure_name};
-    return $self->{'procedures'}{$procedure_name};
+      unless exists $self->_procedures->{$procedure_name};
+    return $self->_procedures->{$procedure_name};
 }
 
-# ----------------------------------------------------------------------
 sub get_procedures {
 
 =pod
@@ -534,7 +508,7 @@ Returns all the procedures as an array or array reference.
     my @procedures =
       map  { $_->[1] }
       sort { $a->[0] <=> $b->[0] }
-      map  { [ $_->order, $_ ] } values %{ $self->{'procedures'} };
+      map  { [ $_->order, $_ ] } values %{ $self->_procedures };
 
     if (@procedures) {
         return wantarray ? @procedures : \@procedures;
@@ -545,7 +519,6 @@ Returns all the procedures as an array or array reference.
     }
 }
 
-# ----------------------------------------------------------------------
 sub get_table {
 
 =pod
@@ -562,18 +535,17 @@ Returns a table by the name provided.
     my $table_name = shift or return $self->error('No table name');
     my $case_insensitive = shift;
     if ( $case_insensitive ) {
-    	$table_name = uc($table_name);
-    	foreach my $table ( keys %{$self->{tables}} ) {
-    		return $self->{tables}{$table} if $table_name eq uc($table);
-    	}
-    	return $self->error(qq[Table "$table_name" does not exist]);
+      $table_name = uc($table_name);
+      foreach my $table ( keys %{$self->_tables} ) {
+         return $self->_tables->{$table} if $table_name eq uc($table);
+      }
+      return $self->error(qq[Table "$table_name" does not exist]);
     }
     return $self->error(qq[Table "$table_name" does not exist])
-      unless exists $self->{'tables'}{$table_name};
-    return $self->{'tables'}{$table_name};
+      unless exists $self->_tables->{$table_name};
+    return $self->_tables->{$table_name};
 }
 
-# ----------------------------------------------------------------------
 sub get_tables {
 
 =pod
@@ -590,7 +562,7 @@ Returns all the tables as an array or array reference.
     my @tables =
       map  { $_->[1] }
       sort { $a->[0] <=> $b->[0] }
-      map  { [ $_->order, $_ ] } values %{ $self->{'tables'} };
+      map  { [ $_->order, $_ ] } values %{ $self->_tables };
 
     if (@tables) {
         return wantarray ? @tables : \@tables;
@@ -601,7 +573,6 @@ Returns all the tables as an array or array reference.
     }
 }
 
-# ----------------------------------------------------------------------
 sub get_trigger {
 
 =pod
@@ -617,11 +588,10 @@ Returns a trigger by the name provided.
     my $self = shift;
     my $trigger_name = shift or return $self->error('No trigger name');
     return $self->error(qq[Table "$trigger_name" does not exist])
-      unless exists $self->{'triggers'}{$trigger_name};
-    return $self->{'triggers'}{$trigger_name};
+      unless exists $self->_triggers->{$trigger_name};
+    return $self->_triggers->{$trigger_name};
 }
 
-# ----------------------------------------------------------------------
 sub get_triggers {
 
 =pod
@@ -638,7 +608,7 @@ Returns all the triggers as an array or array reference.
     my @triggers =
       map  { $_->[1] }
       sort { $a->[0] <=> $b->[0] }
-      map  { [ $_->order, $_ ] } values %{ $self->{'triggers'} };
+      map  { [ $_->order, $_ ] } values %{ $self->_triggers };
 
     if (@triggers) {
         return wantarray ? @triggers : \@triggers;
@@ -649,7 +619,6 @@ Returns all the triggers as an array or array reference.
     }
 }
 
-# ----------------------------------------------------------------------
 sub get_view {
 
 =pod
@@ -665,11 +634,10 @@ Returns a view by the name provided.
     my $self = shift;
     my $view_name = shift or return $self->error('No view name');
     return $self->error('View "$view_name" does not exist')
-      unless exists $self->{'views'}{$view_name};
-    return $self->{'views'}{$view_name};
+      unless exists $self->_views->{$view_name};
+    return $self->_views->{$view_name};
 }
 
-# ----------------------------------------------------------------------
 sub get_views {
 
 =pod
@@ -686,7 +654,7 @@ Returns all the views as an array or array reference.
     my @views =
       map  { $_->[1] }
       sort { $a->[0] <=> $b->[0] }
-      map  { [ $_->order, $_ ] } values %{ $self->{'views'} };
+      map  { [ $_->order, $_ ] } values %{ $self->_views };
 
     if (@views) {
         return wantarray ? @views : \@views;
@@ -697,7 +665,6 @@ Returns all the views as an array or array reference.
     }
 }
 
-# ----------------------------------------------------------------------
 sub make_natural_joins {
 
 =pod
@@ -711,14 +678,14 @@ tables.  Accepts the following arguments:
 
 =item * join_pk_only
 
-A True or False argument which determins whether or not to perform 
+A True or False argument which determins whether or not to perform
 the joins from primary keys to fields of the same name in other tables
 
 =item * skip_fields
 
 A list of fields to skip in the joins
 
-=back 4
+=back
 
   $schema->make_natural_joins(
       join_pk_only => 1,
@@ -769,11 +736,6 @@ A list of fields to skip in the joins
     return 1;
 }
 
-# ----------------------------------------------------------------------
-sub name {
-
-=pod
-
 =head2 name
 
 Get or set the schema's name.  (optional)
@@ -782,13 +744,7 @@ Get or set the schema's name.  (optional)
 
 =cut
 
-    my $self = shift;
-    $self->{'name'} = shift if @_;
-    return $self->{'name'} || '';
-}
-
-# ----------------------------------------------------------------------
-sub translator {
+has name => ( is => 'rw', default => quote_sub(q{ '' }) );
 
 =pod
 
@@ -798,22 +754,10 @@ Get the SQL::Translator instance that instantiated the parser.
 
 =cut
 
-    my $self = shift;
-    $self->{'translator'} = shift if @_;
-    return $self->{'translator'};
-}
-
-# ----------------------------------------------------------------------
-sub DESTROY {
-    my $self = shift;
-    undef $_ for values %{ $self->{'tables'} };
-    undef $_ for values %{ $self->{'views'} };
-}
+has translator => ( is => 'rw', weak_ref => 1 );
 
 1;
 
-# ----------------------------------------------------------------------
-
 =pod
 
 =head1 AUTHOR
